#!/bin/sh

####################
# manage a set of simpler (than git-miror.sh does), pull-only repos
# the idea here is, we need to keep a local read-only mirror
# of some repositories
#
# e.g. /git-slave/tophat.git on onelab.eu which is a mirror of
# git.top-hat.info/tophat.git that can't easily be exposed as a git feed
#
# instead of hacking git-mirror.sh script that is already scary, 
# I preferred to keep this separate
#
# so the overall layout is as follows
# * manually created the local mirror by running
#   cd /git-slave
#   git clone --mirror ssh://tophat@git.top-hat.info/tophat.git
#   which creates a *bare* repo /git-slave/tophat.git
# * create a symlink for git-daemon and for gitweb
#   cd /git
#   ln -s /git-slave/tophat.git
# Being a symlink this won't be considered by the main purpose of git-mirror.sh 

# Typical usage in cron
# */3 * * * * /root/bin/git-slave.sh -q /git-slave/*.git

# you might wish to tweak the repo's config, in particular the remote section
# so as to define the proper mappings for your needs
# e.g. from git.onelab.eu:/git-slave/rvm-rpm.git
#[remote "origin"]
#	url = https://github.com/cdwertmann/rvm-rpm.git
#	fetch = refs/heads/*:refs/heads/*

COMMAND=$(basename $0)

function slave_repo () {
    git_slave=$1; shift
    cd $git_slave
    git fetch --all --tags -q || echo "$COMMAND failed in repo $git_slave"
}

for slave_dir in "$@"; do slave_repo $slave_dir; done
